﻿QuestProgressSound = LibStub( "AceAddon-3.0"):NewAddon( "QuestProgressSound", "AceConsole-3.0", "AceEvent-3.0", "LibSink-2.0")
local Quixote = LibStub("LibQuixote-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("QuestProgressSound")

local sounds = {
	soundObjectiveUpdate = {
		"Sound\\Creature\\Peon\\PeonBuildingComplete1.wav",
		"Sound\\Creature\\Peasant\\PeasantWhat3.wav",
	},
	soundQuestComplete = {
		"Sound\\Creature\\Peon\\PeonReady1.wav",
		"Interface\\Addons\\QuestProgressSound\\Sound\\Peasant_work_done.mp3",
	},
	soundparty = {
		"Interface\\Addons\\QuestProgressSound\\Sound\\ding.mp3",
		"Interface\\Addons\\QuestProgressSound\\Sound\\tada.mp3",
		"Interface\\Addons\\QuestProgressSound\\Sound\\Alert.mp3",
		"Interface\\Addons\\QuestProgressSound\\Sound\\Info.mp3",
	},
}

local soundnames = {
	soundObjectiveUpdate = {
		"Peon - Complete",
		"Peasant - More Work",
	},
	soundQuestComplete = {
		"Peon - Ready",
		"Peasant - Job's done",
	},
	soundparty = {
		"Ding",
		"Tada",
		"Alert",
		"Info",
	},
}

local options = {
	name = "QuestProgressSound",
	handler = QuestProgressSound,
	type = "group",
	childGroups = "tab",
	get = "getValue",
	set = "setValue",
	args = {
		player = {
			name = L["Player Sounds"],
			type = "group",
			args = {
				enableSound = {
					name = L["Enable Sound"],
					desc = L["Enables / disables sound"],
					type = "toggle",
				},
				soundObjectiveUpdate = {
					name = L["Object Sound"],
					desc = L["Sound played when Part of Quest completed"],
					type = "select",
					values = soundnames.soundObjectiveUpdate,
					set = "setSound",
				},
				soundQuestComplete = {
					name = L["Quest Complete Sound"],
					desc = L["Sound played when all quest objects completed"],
					type = "select",
					values = soundnames.soundQuestComplete,
					set = "setSound",
				},
			},
		},
		party = {
			name = L["Party Sounds"],
			type = "group",
			args = {
				enablePartyQuestProgress = {
					name = L["Enable Party Progress Sound"],
					desc = L["Plays defined sound when someone in your party fulfilled a part of a quest"],
					type = "toggle",
				},
				enablePartyQuestComplete = {
					name = L["Enable Party Quest Complete"],
					desc = L["Plays defined sound when someone in your party completes a quest"],
					type = "toggle",
				},
				soundpartyprogress = {
					name = L["Party Progress Sound"],
					desc = L["Sound played when Part of Quest completed by someone in your party"],
					type = "select",
					values = soundnames.soundparty,
					set = "setSoundParty",
				},
				soundpartycomplete = {
					name = L["Party Quest Complete Sound"],
					desc = L["Sound played when all quest objects completed by someone in your party"],
					type = "select",
					values = soundnames.soundparty,
					set = "setSoundParty",
				},
			},
		},
		textnotify = {
			name = "Visual Party Progress",
			type = "group",
			args = {
				enableOutput = {
					name = L["Enable Output"],
					desc = L["Shows progress of party members"],
					type = "toggle",
				},
				output = QuestProgressSound:GetSinkAce3OptionsDataTable(),
			},
		},
	},
}

local defaults = {
	profile = {
		enableSound = true,
		enableOutput = false,
		sinkoptions = {},
		soundObjectiveUpdate = 1,
		soundQuestComplete = 1,
		enablePartyQuestProgress = false,
		enablePartyQuestComplete = false,
		soundpartyprogress = 1,
		soundpartycomplete = 1,
	}
}

function QuestProgressSound:getValue(info)
	return self.db.profile[info[#info]]
end

function QuestProgressSound:setValue(info,value)
	self.db.profile[info[#info]] = value
end

function QuestProgressSound:setSound(info,value)
	self.db.profile[info[#info]] = value
	PlaySoundFile(sounds[info[#info]][value])
end

function QuestProgressSound:setSoundParty(info,value)
	self.db.profile[info[#info]] = value
	PlaySoundFile(sounds.soundparty[value])
end

function QuestProgressSound:OnInitialize()
	-- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("QuestProgressSoundDB", defaults, "Default")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	self:SetSinkStorage(self.db.profile.sinkoptions)
	
	options.args.textnotify.args.output.disabled = function() return not self.db.profile.enableOutput end
	options.args.party.args.soundpartyprogress.disabled = function() return not self.db.profile.enablePartyQuestProgress end
	options.args.party.args.soundpartycomplete.disabled = function() return not self.db.profile.enablePartyQuestComplete end
	options.args.player.args.soundObjectiveUpdate.disabled = function() return not self.db.profile.enableSound end
	options.args.player.args.soundQuestComplete.disabled = function() return not self.db.profile.enableSound end
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuestProgressSound", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuestProgressSound","QuestProgressSound")
	self:RegisterChatCommand("qps", "ChatCommand")
end

function QuestProgressSound:OnEnable()
	-- Called when the addon is enabled
	self:Print(L["Quest Progress Sounds loaded"])
	Quixote.RegisterCallback(self, "Objective_Update")
	Quixote.RegisterCallback(self, "Quest_Complete")
	Quixote.RegisterCallback(self, "Party_Objective_Update")
	Quixote.RegisterCallback(self, "Party_Quest_Complete")
end

function QuestProgressSound:Objective_Update(eventname, questtitle,uid,desc,w,currentobjects,neededobjects,type)
	if (self.db.profile.enableSound) and (currentobjects==neededobjects) then
		PlaySoundFile(sounds.soundObjectiveUpdate[self.db.profile.soundObjectiveUpdate])
	end
end

function QuestProgressSound:Quest_Complete()
	if (self.db.profile.enableSound) then
		PlaySoundFile(sounds.soundQuestComplete[self.db.profile.soundQuestComplete])
	end
end

function QuestProgressSound:Party_Objective_Update(event, sender, uid, questtitle, objective, numGot, numNeeded)
	if (self.db.profile.enableOutput) then
		self:Pour(L["Questprogress of "]..sender..": "..questtitle.." "..numGot.."/"..numNeeded.." "..objective)
	end
	if (self.db.profile.enablePartyQuestProgress) and (numGot==numNeeded) then
		PlaySoundFile(sounds.soundparty[self.db.profile.soundpartyprogress])
	end
end

function QuestProgressSound:Party_Quest_Complete(event, sender, uid, questtitle)
	if (self.db.profile.enableOutput) then
		self:Pour(sender.." "..L["completed"].." "..questtitle)
	end
	if (self.db.profile.enablePartyQuestComplete) then
		PlaySoundFile(sounds.soundparty[self.db.profile.soundpartycomplete])
	end
end

function QuestProgressSound:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end